@@ -17,43 +17,22 @@ use constant {
     FALSE => 0,
     TRUE  => 1,
     DEBUG => $ENV{ DEBUG } // 0,
+
+    FILE_ALREADY_DOWNLOADED => -1,
 };
 
 use Term::ProgressBar;
 
-#-------------------------------------------------------------------------------
-# Tipos de protocolos suportados
-#-------------------------------------------------------------------------------
-my $supportedProtocols = {
-    'FTP'   => TRUE,
-    'SFTP'  => TRUE,
-    'LOCAL' => TRUE
-};
-
-#-------------------------------------------------------------------------------
-# Tipos de MIMI-Types suportados
-#-------------------------------------------------------------------------------
-my $supported_mimi_types = {
-    'application/x-tar-gz'         => FALSE,          # .tar.gz
-    'application/x-bzip'           => FALSE,          # .bz2
-    'application/zip'              => 'unZipFile',    # .zip
-    'application/x-rar-compressed' => FALSE,          # .rar
-    'application/x-tar'            => FALSE,          # .tar
-    'application/pdf'              => FALSE,          # .pdf
-                                                      # este não existe MIME Type, mas usem este:
-    'deltax/tar-bz2'               => FALSE,          # tar.bz2
-};
-
 BEGIN
 {
     binmode( STDOUT, ":encoding(UTF-8)" );
     binmode( STDERR, ":encoding(UTF-8)" );
 
-    require Siffra::Base;
+    require Siffra::Tools;
     use Exporter ();
     use vars qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);
-    $VERSION = '0.03';
-    @ISA     = qw(Siffra::Base Exporter);
+    $VERSION = '0.04';
+    @ISA     = qw(Siffra::Tools Exporter);
 
     #Give a hoot don't pollute, do not export more than needed by default
     @EXPORT      = qw();
@@ -83,6 +62,17 @@ sub new
 
     my $self = $class->SUPER::new( %parameters );
 
+    $self->_initialize( %parameters );
+
+    return $self;
+} ## end sub new
+
+sub _initialize()
+{
+    my ( $self, %parameters ) = @_;
+    $self->SUPER::_initialize( %parameters );
+    $log->debug( "_initialize", { package => __PACKAGE__ } );
+
     $self->{ config } = {
         protocol       => undef,
         host           => undef,
@@ -99,17 +89,39 @@ sub new
     $self->{ connection } = undef;
     $self->{ json }       = undef;
 
-    $self->_initialize( %parameters );
+    #-------------------------------------------------------------------------------
+    # Tipos de protocolos suportados
+    #-------------------------------------------------------------------------------
+    $self->{ supportedProtocols } = {
+        'FTP' => {
+            connect  => 'connectFTP',
+            getFiles => 'getFilesFTP',
+        },
+        'SFTP' => {
+            connect  => 'connectSFTP',
+            getFiles => 'getFilesSFTP',
+        },
+        'LOCAL' => {
+            connect  => 'connectLOCAL',
+            getFiles => 'getFilesLOCAL',
+        },
+    };
 
-    return $self;
-} ## end sub new
-
-sub _initialize()
-{
-    my ( $self, %parameters ) = @_;
-    $self->SUPER::_initialize( %parameters );
-    $log->debug( "_initialize", { package => __PACKAGE__ } );
-}
+    #-------------------------------------------------------------------------------
+    # Tipos de MIME-Types suportados
+    #-------------------------------------------------------------------------------
+    $self->{ supportedMimeTypes } = {
+        'application/x-tar-gz'         => FALSE,          # .tar.gz
+        'application/x-gzip'           => FALSE,          # .tar.gz
+        'application/x-bzip'           => FALSE,          # .bz2
+        'application/x-bzip2'          => FALSE,          # .bz2
+        'application/zip'              => 'unZipFile',    # .zip
+        'application/x-rar-compressed' => FALSE,          # .rar
+        'application/x-tar'            => FALSE,          # .tar
+        'application/x-7z-compressed'  => FALSE,          # .tar
+        'application/pdf'              => FALSE,          # .pdf
+    };
+} ## end sub _initialize
 
 sub END
 {
@@ -121,7 +133,13 @@ sub END
 
 my $setProtocol = sub {
     my ( $self, $value ) = @_;
-    return FALSE unless ( $supportedProtocols->{ uc( $value ) } );
+
+    if ( !$self->{ supportedProtocols }->{ uc( $value ) } )
+    {
+        $log->error( "Protocolo [ $value ] não suportado..." );
+        return FALSE;
+    }
+
     return $self->{ config }->{ protocol } = uc( $value );
 };
 my $setHost = sub {
@@ -148,6 +166,10 @@ my $setPassive = sub {
     my ( $self, $value ) = @_;
     return $self->{ config }->{ passive } = $value;
 };
+my $setSsl = sub {
+    my ( $self, $value ) = @_;
+    return $self->{ config }->{ ssl } = $value;
+};
 my $setIdentityFile = sub {
     my ( $self, $value ) = @_;
     return $self->{ config }->{ identity_file } = $value;
@@ -159,6 +181,7 @@ my $setLocalDirectory = sub {
 my $setDirectories = sub {
     my ( $self, $value ) = @_;
 
+    $self->{ config }->{ directories } = undef;
     while ( my ( $remoteDirectory, $configuration ) = each( %{ $value } ) )
     {
         $configuration->{ remoteDirectory } = $remoteDirectory;
@@ -178,9 +201,20 @@ my $setDirectories = sub {
 sub addDirectory
 {
     my ( $self, $configuration ) = @_;
+
+    return FALSE unless $configuration->{ fileNameRule };
+    return FALSE unless $configuration->{ remoteDirectory };
+    return FALSE unless ref $configuration->{ downloadedFiles } eq 'HASH';
+
+    #{
+    #    downloadedFiles   {},
+    #    fileNameRule      "VALID_RETORNO_.*",
+    #    remoteDirectory   "/valid/upload/"
+    #}
+
     $self->{ config }->{ directories }->{ $configuration->{ remoteDirectory } } = $configuration;
     return TRUE;
-}
+} ## end sub addDirectory
 
 =head2 C<cleanDirectories()>
 =cut
@@ -208,6 +242,7 @@ sub setConfig()
     $self->$setPort( $parameters{ port } );
     $self->$setDebug( $parameters{ debug } );
     $self->$setPassive( $parameters{ passive } );
+    $self->$setSsl( $parameters{ ssl } );
     $self->$setIdentityFile( $parameters{ identity_file } );
     $self->$setLocalDirectory( $parameters{ localDirectory } );
     $self->$setDirectories( $parameters{ directories } );
@@ -234,36 +269,25 @@ sub connect()
 
     return FALSE unless $self->testConfig();
 
-    my $retorno = FALSE;
+    my $connectSub = $self->{ supportedProtocols }->{ uc $self->{ config }->{ protocol } }->{ connect };
 
-    if ( uc $self->{ config }->{ protocol } eq 'FTP' )
-    {
-        $retorno = $self->connectFTP( %parameters );
-    }
-    elsif ( uc $self->{ config }->{ protocol } eq 'SFTP' )
-    {
-        $retorno = $self->connectSFTP( %parameters );
-    }
-    elsif ( uc $self->{ config }->{ protocol } eq 'LOCAL' )
-    {
-        $retorno = $self->connectLocal( %parameters );
-    }
-
-    return $retorno;
+    return $self->$connectSub( %parameters );
 } ## end sub connect
 
 =head2 C<connectFTP()>
 =cut
 
-sub connectFTP()
+sub connectFTP ()
 {
     my ( $self, %parameters ) = @_;
 
-    eval { require Net::FTP; };
+    my $moduleConn = 'Net::FTP';
+
+    eval "require $moduleConn;";
 
     if ( $@ )
     {
-        $log->error( "Erro ao usar o módulo [ Net::FTP ]..." . $@ );
+        $log->error( "Erro ao usar o módulo [ $moduleConn ]..." . $@ );
         return FALSE;
     }
 
@@ -274,16 +298,12 @@ sub connectFTP()
         Passive => $self->{ config }->{ passive } // 1,
     );
 
+    $log->info( "Conectando no FTP [ $args{Host}\:$args{Port} ]" );
     $self->{ connection } = Net::FTP->new( %args );
 
-    if ( $@ )
-    {
-        $log->error( "Erro ao conectar no FTP [ $args{Host}\:$args{Port} ] ... [ $@ ]" );
-        return FALSE;
-    }
-
     if ( $self->{ connection } )
     {
+        $self->{ connection }->starttls() if $self->{ config }->{ ssl };
         my $user     = $self->{ config }->{ user };
         my $password = $self->{ config }->{ password };
         if ( !$self->{ connection }->login( $user, $password ) )
@@ -293,30 +313,33 @@ sub connectFTP()
         }
         else
         {
-            $log->info( "Logged on [ ${user}\@$args{Host} ]..." );
-            return TRUE;
+            $log->info( "Conexão feita com sucesso no FTP [ ${user}\@$args{Host} ]..." );
+            return $self->{ connection };
         }
-
     } ## end if ( $self->{ connection...})
     else
     {
         $log->error( "Não foi possível criar o objeto FTP..." );
         return FALSE;
     }
+
+    return FALSE;
 } ## end sub connectFTP
 
 =head2 C<connectSFTP()>
 =cut
 
-sub connectSFTP()
+sub connectSFTP ()
 {
     my ( $self, %parameters ) = @_;
 
-    eval { require Net::SFTP::Foreign; };
+    my $moduleConn = 'Net::SFTP::Foreign';
+
+    eval "require $moduleConn;";
 
     if ( $@ )
     {
-        $log->error( "Erro ao usar o módulo [ Net::SFTP::Foreign ]..." . $@ );
+        $log->error( "Erro ao usar o módulo [ $moduleConn ]..." . $@ );
         return FALSE;
     }
 
@@ -335,26 +358,32 @@ sub connectSFTP()
     push @{ $args{ key_path } }, $self->{ config }->{ identity_file } if $self->{ config }->{ identity_file };
     push @{ $args{ more } }, '-v' if ( $self->{ config }->{ debug } );
 
+    $log->info( "Conectando no SFTP [ $args{host}\:$args{port} ]" );
     $self->{ connection } = eval { Net::SFTP::Foreign->new( %args ); };
 
     if ( $@ )
     {
         $log->error( $@ );
+        $log->error( "Não foi possível criar o objeto SFTP..." );
         return FALSE;
-    }
+    } ## end if ( $@ )
     elsif ( $self->{ connection }->error )
     {
         $log->error( $self->{ connection }->error );
         return FALSE;
     }
+    else
+    {
+        $log->info( "Conexão feita com sucesso no SFTP [ $args{user}\@$args{host} ]..." );
+    }
 
-    return TRUE;
+    return $self->{ connection };
 } ## end sub connectSFTP
 
 =head2 C<connectLocal()>
 =cut
 
-sub connectLocal()
+sub connectLocal ()
 {
     my ( $self, %parameters ) = @_;
 
@@ -398,32 +427,15 @@ sub getFiles
 
     return FALSE unless ( $self->testConfig() );
 
-    my $getFilesFunctions = {
-        'FTP'   => 'getFilesFTP',
-        'SFTP'  => 'getFilesSFTP',
-        'LOCAL' => 'getFilesLOCAL'
-    };
-
-    my $protocol = uc $self->{ config }->{ protocol };
-    my $subName;
-
-    if ( !$getFilesFunctions->{ $protocol } )
-    {
-        $log->error( "SUB getFiles para o protocolo [ $protocol ] não existe." );
-        return FALSE;
-    }
-    else
-    {
-        $subName = $getFilesFunctions->{ $protocol };
-    }
-
-    my $retorno = {};
+    my $protocol    = uc $self->{ config }->{ protocol };
+    my $getFilesSub = $self->{ supportedProtocols }->{ $protocol }->{ getFiles };
+    my $retorno;
 
     while ( my ( $directory, $directoryConfiguration ) = each( %{ $self->{ config }->{ directories } } ) )
     {
         $self->setActiveDirectory( $directory );
 
-        $retorno->{ $directory } = $self->$subName( %parameters );
+        $retorno->{ $directory } = $self->$getFilesSub( %parameters );
 
         if ( ( ref $retorno->{ $directory } eq 'HASH' ) && ( $retorno->{ $directory }->{ error } == 0 ) && ( $directoryConfiguration->{ 'unpack' } ) )
         {
@@ -448,10 +460,129 @@ sub getFiles
 
 sub getFilesFTP()
 {
+    $log->debug( "getFilesFTP", { package => __PACKAGE__ } );
     my ( $self, %parameters ) = @_;
 
-    return TRUE;
-}
+    my $retorno = {
+        error              => 0,
+        message            => 'ok',
+        downloadedFiles    => [],
+        notDownloadedFiles => []
+    };
+
+    my $remoteDirectory = $self->getActiveDirectory();
+    my $configuration   = $self->{ config }->{ directories }->{ $remoteDirectory };
+    my $localDirectory  = $self->{ config }->{ localDirectory };
+
+    $log->info( "Entrando em [ getFilesFTP ] para o diretório [ $remoteDirectory ]..." );
+
+    unless ( $self->{ connection }->cwd( $remoteDirectory ) )
+    {
+        return {
+            error   => 1,
+            message => $self->{ connection }->message()
+        };
+    } ## end unless ( $self->{ connection...})
+
+    unless ( $self->{ connection }->binary() )
+    {
+        return {
+            error   => 1,
+            message => $self->{ connection }->message()
+        };
+    } ## end unless ( $self->{ connection...})
+
+    my $remoteFiles = $self->{ connection }->ls();
+
+    if ( ( !defined $remoteFiles ) && ( $self->{ connection }->message() =~ /No files found/ ) )
+    {
+        return {
+            error   => 0,
+            message => $self->{ connection }->message(),
+            files   => []
+        };
+    } ## end if ( ( !defined $remoteFiles...))
+    elsif ( !defined $remoteFiles )
+    {
+        return {
+            error   => 1,
+            message => ( $self->{ connection }->message() // '' )
+        };
+    } ## end elsif ( !defined $remoteFiles...)
+    else
+    {
+        foreach my $remoteFile ( @{ $remoteFiles } )
+        {
+            my $status = $self->canDownloadFile( fileName => $remoteFile );
+            if ( $status < 0 )
+            {
+                push( @{ $retorno->{ notDownloadedFiles } }, { name => $remoteFile, status => $status } );
+                next;
+            }
+
+            $log->info( "Baixando o arquivo [ $remoteFile ]..." );
+
+            my $localFile = $localDirectory . '/' . $remoteFile;
+            my $get       = eval { $self->{ connection }->get( $remoteFile, $localFile ); };
+
+            my $file = {
+                error     => 0,
+                message   => '',
+                name      => $remoteFile,
+                file_size => -1,
+                md5sum    => undef,
+                filePath  => $localFile
+            };
+
+            if ( !$get )
+            {
+                $file->{ error }   = 1;
+                $file->{ message } = $self->{ connection }->message();
+            }
+            else
+            {
+                $file->{ file_size } = -s $localFile;
+                my $md5 = $self->getFileMD5( file => $localFile );
+                $file->{ md5sum } = $md5;
+            } ## end else [ if ( !$get ) ]
+
+            push( @{ $retorno->{ downloadedFiles } }, $file );
+            last() if ( $parameters{ only_one_file } );
+        } ## end foreach my $remoteFile ( @{...})
+    } ## end else [ if ( ( !defined $remoteFiles...))]
+
+    return $retorno;
+} ## end sub getFilesFTP
+
+=head2 C<canDownloadFile()>
+=cut
+
+sub canDownloadFile()
+{
+    $log->debug( "canDownloadFile", { package => __PACKAGE__ } );
+    my ( $self, %parameters ) = @_;
+    my $fileName = $parameters{ fileName };
+
+    my $activeDownload = $self->{ config }->{ directories }->{ $self->getActiveDirectory() };
+
+    # ja foi baixado
+    return -1 if ( $activeDownload->{ downloadedFiles }{ $fileName } );
+
+    # nao bate com a regra de nomes
+    if ( defined $activeDownload->{ fileNameRule } )
+    {
+        $self->{ message } = "Arquivo [ $fileName ] não bate com a regra de filename_pattern.";
+        return -2 if ( $fileName !~ /$activeDownload->{fileNameRule}/ );
+        return TRUE;
+    } ## end if ( defined $activeDownload...)
+    else
+    {
+        $log->error( "Não existe regra para o nome do arquivo......" );
+        return FALSE;
+    }
+
+    return FALSE;
+} ## end sub canDownloadFile
 
 =head2 C<getFilesSFTP()>
 =cut
@@ -481,51 +612,62 @@ sub getFilesSFTP()
         }
     );
 
-    my $callback = sub {
-        my ( $sftp, $data, $offset, $size, $progress ) = @_;
-        $offset = $size if ( $offset >= $size );
-        $progress->update( $offset );
-    };
-
-    foreach my $remoteFile ( @{ $ls } )
+    if ( $ls )
     {
-        my $progress = Term::ProgressBar->new(
-            {
-                name   => $remoteFile->{ filename } . " ( $remoteFile->{ a }->{ size } ) ",
-                count  => $remoteFile->{ a }->{ size },
-                ETA    => 'linear',
-                remove => 1,
-            }
-        );
-        $progress->minor( 0 );
-
-        my %options = ( callback => sub { &$callback( @_, $progress ); }, mkpath => 1 );
-        $self->{ connection }->get( $remoteDirectory . $remoteFile->{ filename }, $localDirectory . $remoteFile->{ filename }, %options );
-
-        my $downloadedFile = {
-            error    => FALSE,
-            message  => undef,
-            filename => $remoteFile->{ filename },
-            size     => $remoteFile->{ a }->{ size },
-            md5sum   => undef,
-            filePath => $localDirectory . $remoteFile->{ filename },
+        my $callback = sub {
+            my ( $sftp, $data, $offset, $size, $progress ) = @_;
+            $offset = $size if ( $offset >= $size );
+            $progress->update( $offset );
         };
 
-        if ( $self->{ connection }->error )
+        foreach my $remoteFile ( @{ $ls } )
         {
-            $log->error( $self->{ connection }->error );
-            $downloadedFile->{ error }   = TRUE;
-            $downloadedFile->{ message } = $self->{ connection }->error;
-        } ## end if ( $self->{ connection...})
-        else
-        {
-            $downloadedFile->{ md5sum } = $self->getFileMD5( file => $localDirectory . $remoteFile->{ filename } );
-            $downloadedFile->{ message } = 'Ok';
-            $log->info( "Download do arquivo [ $downloadedFile->{ filename } ] feito com sucesso..." );
-        } ## end else [ if ( $self->{ connection...})]
+            my $progress = Term::ProgressBar->new(
+                {
+                    name   => $remoteFile->{ filename } . " ( $remoteFile->{ a }->{ size } ) ",
+                    count  => $remoteFile->{ a }->{ size },
+                    ETA    => 'linear',
+                    remove => 1,
+                }
+            );
+            $progress->minor( 0 );
 
-        push( @{ $retorno->{ downloadedFiles } }, $downloadedFile );
-    } ## end foreach my $remoteFile ( @{...})
+            my %options = ( callback => sub { &$callback( @_, $progress ); }, mkpath => 1 );
+
+            #$log->debug( $remoteDirectory . $remoteFile->{ filename } );
+            $self->{ connection }->get( $remoteDirectory . $remoteFile->{ filename }, $localDirectory . $remoteFile->{ filename }, %options );
+
+            my $downloadedFile = {
+                error    => FALSE,
+                message  => undef,
+                filename => $remoteFile->{ filename },
+                size     => $remoteFile->{ a }->{ size },
+                md5sum   => undef,
+                filePath => $localDirectory . $remoteFile->{ filename },
+            };
+
+            if ( $self->{ connection }->error )
+            {
+                $log->error( $self->{ connection }->error );
+                $log->error( $remoteDirectory . $remoteFile->{ filename } );
+                $downloadedFile->{ error }      = TRUE;
+                $downloadedFile->{ message }    = $self->{ connection }->error;
+                $downloadedFile->{ remoteFile } = $remoteDirectory . $remoteFile->{ filename };
+            } ## end if ( $self->{ connection...})
+            else
+            {
+                $downloadedFile->{ md5sum }  = $self->getFileMD5( file => $localDirectory . $remoteFile->{ filename } );
+                $downloadedFile->{ message } = 'Ok';
+                $log->info( "Download do arquivo [ $downloadedFile->{ filename } ] feito com sucesso..." );
+            } ## end else [ if ( $self->{ connection...})]
+
+            push( @{ $retorno->{ downloadedFiles } }, $downloadedFile );
+        } ## end foreach my $remoteFile ( @{...})
+    } ## end if ( $ls )
+    else
+    {
+        $log->warn( 'Nenhuma arquivo para ser baixado...' );
+    }
 
     return $retorno;
 } ## end sub getFilesSFTP
@@ -539,32 +681,6 @@ sub getFilesLOCAL()
     return TRUE;
 }
 
-=head2 C<getFileMD5()>
-=cut
-
-#-------------------------------------------------------------------------------
-# Retorna o MD5 do arquivo
-# Parametro 1 - Caminho e nome do arquivo a ser calculado
-# Retorna o MD5 do arquivo informado
-#-------------------------------------------------------------------------------
-sub getFileMD5()
-{
-    my ( $self, %parameters ) = @_;
-    my $file = $parameters{ file };
-    my $retorno;
-
-    return FALSE unless ( -e $file );
-
-    require Digest::MD5;
-
-    open( my $fh, '<', $file ) or croak( "Can't open '$file': $!" );
-    binmode( $fh );
-    $retorno = Digest::MD5->new->addfile( $fh )->hexdigest;
-    close( $fh );
-
-    return $retorno;
-} ## end sub getFileMD5
-
 #-------------------------------------------------------------------------------
 # Descompacta arquivos
 #-------------------------------------------------------------------------------
@@ -575,23 +691,23 @@ sub getFileMD5()
 sub unPackFile()
 {
     my ( $self, %parameters ) = @_;
-    my $mmt = $parameters{ conf }{ 'MIMI-Type' };
+    my $mmt = $parameters{ conf }{ 'MIME-Type' };
 
     my $files = [];
-    if ( !$supported_mimi_types->{ $mmt } == FALSE )
+    if ( !$self->{ supportedMimeTypes }->{ $mmt } == FALSE )
     {
         my $unPackParam = {
             fileName => $parameters{ file }{ file_path },
             out_path => $parameters{ conf }{ unpackDirectory }
         };
 
-        my $subname = $supported_mimi_types->{ $mmt };
+        my $subname = $self->{ supportedMimeTypes }->{ $mmt };
         $files = $self->$subname( $unPackParam );
 
-    } ## end if ( !$supported_mimi_types...)
+    } ## end if ( !$self->{ supportedMimeTypes...})
     else
     {
-        return { message => "MIMI-Type $mmt não suportado", error => 1 };
+        return { message => "MIME-Type $mmt não suportado", error => 1 };
     }
 
     return { error => 0, files => $files };
